==> fastutil-7.0.13-sources.jar: it/unimi/dsi/fastutil/bytes/AbstractByte2ObjectMap.java <==
/* Generic definitions */

/* Assertions (useful to generate conditional code) */

/* Current type and class (and size, if applicable) */
/* Value methods */

/* Interfaces (keys) */
/* Interfaces (values) */
/* Abstract implementations (keys) */
/* Abstract implementations (values) */

/* Static containers (keys) */
/* Static containers (values) */

/* Implementations */
/* Synchronized wrappers */
/* Unmodifiable wrappers */
/* Other wrappers */

/* Methods (keys) */
/* Methods (values) */
/* Methods (keys/values) */

/* Methods that have special names depending on keys (but the special names depend on values) */

/* Equality */
/* Object/Reference-only definitions (keys) */
/* Primitive-type-only definitions (keys) */
/* Object/Reference-only definitions (values) */
/*		 
 * Copyright (C) 2002-2016 Sebastiano Vigna
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package it.unimi.dsi.fastutil.bytes;

import it.unimi.dsi.fastutil.objects.ObjectCollection;
import it.unimi.dsi.fastutil.objects.AbstractObjectCollection;
import it.unimi.dsi.fastutil.objects.ObjectIterator;
import it.unimi.dsi.fastutil.objects.AbstractObjectIterator;

import it.unimi.dsi.fastutil.objects.ObjectSet;

import java.util.Iterator;
import java.util.Map;

/**
 * An abstract class providing basic methods for maps implementing a
 * type-specific interface.
 *
 * <P>
 * Optional operations just throw an {@link UnsupportedOperationException}.
 * Generic versions of accessors delegate to the corresponding type-specific
 * counterparts following the interface rules (they take care of returning
 * <code>null</code> on a missing key).
 *
 * <P>
 * As a further help, this class provides a {@link BasicEntry BasicEntry} inner
 * class that implements a type-specific version of {@link java.util.Map.Entry};
 * it is particularly useful for those classes that do not implement their own
 * entries (e.g., most immutable maps).
 */

public abstract class AbstractByte2ObjectMap<V>
		extends
			AbstractByte2ObjectFunction<V>
		implements
			Byte2ObjectMap<V>,
			java.io.Serializable {

	private static final long serialVersionUID = -4940583368468432370L;

	protected AbstractByte2ObjectMap() {
	}
	/** Checks whether the given value is contained in {@link #values()}. */
	public boolean containsValue(Object v) {
		return values().contains(v);
	}

	/** Checks whether the given value is contained in {@link #keySet()}. */
	public boolean containsKey(byte k) {
		return keySet().contains(k);
	}

	/**
	 * Puts all pairs in the given map. If the map implements the interface of
	 * this map, it uses the faster iterators.
	 *
	 * @param m
	 *            a map.
	 */

	@SuppressWarnings({"unchecked", "deprecation"})
	public void putAll(Map<? extends Byte, ? extends V> m) {
		int n = m.size();
		final Iterator<? extends Map.Entry<? extends Byte, ? extends V>> i = m
				.entrySet().iterator();

		if (m instanceof Byte2ObjectMap) {
			Byte2ObjectMap.Entry<? extends V> e;
			while (n-- != 0) {
				e = (Byte2ObjectMap.Entry<? extends V>) i.next();
				put(e.getByteKey(), e.getValue());
			}
		} else {
			Map.Entry<? extends Byte, ? extends V> e;
			while (n-- != 0) {
				e = i.next();
				put(e.getKey(), e.getValue());
			}
		}
	}

	public boolean isEmpty() {
		return size() == 0;
	}

	/**
	 * This class provides a basic but complete type-specific entry class for
	 * all those maps implementations that do not have entries on their own
	 * (e.g., most immutable maps).
	 *
	 * <P>
	 * This class does not implement
	 * {@link java.util.Map.Entry#setValue(Object) setValue()}, as the
	 * modification would not be reflected in the base map.
	 */

	public static class BasicEntry<V> implements Byte2ObjectMap.Entry<V> {
		protected byte key;
		protected V value;

		public BasicEntry(final Byte key, final V value) {
			this.key = ((key).byteValue());
			this.value = (value);
		}

		public BasicEntry(final byte key, final V value) {
			this.key = key;
			this.value = value;
		}

		/**
		 * {@inheritDoc}
		 * 
		 * @deprecated Please use the corresponding type-specific method
		 *             instead.
		 */
		@Deprecated
		public Byte getKey() {
			return (Byte.valueOf(key));
		}

		public byte getByteKey() {
			return key;
		}

		public V getValue() {
			return (value);
		}

		public V setValue(final V value) {
			throw new UnsupportedOperationException();
		}
		public boolean equals(final Object o) {
			if (!(o instanceof Map.Entry))
				return false;
			final Map.Entry<?, ?> e = (Map.Entry<?, ?>) o;

			if (e.getKey() == null || !(e.getKey() instanceof Byte))
				return false;

			return ((key) == (((((Byte) (e.getKey())).byteValue()))))
					&& ((value) == null ? ((e.getValue())) == null : (value)
							.equals((e.getValue())));
		}

		public int hashCode() {
			return (key) ^ ((value) == null ? 0 : (value).hashCode());
		}

		public String toString() {
			return key + "->" + value;
		}
	}

	/**
	 * Returns a type-specific-set view of the keys of this map.
	 *
	 * <P>
	 * The view is backed by the set returned by {@link #entrySet()}. Note that
	 * <em>no attempt is made at caching the result of this method</em>, as this
	 * would require adding some attributes that lightweight implementations
	 * would not need. Subclasses may easily override this policy by calling
	 * this method and caching the result, but implementors are encouraged to
	 * write more efficient ad-hoc implementations.
	 *
	 * @return a set view of the keys of this map; it may be safely cast to a
	 *         type-specific interface.
	 */

	public ByteSet keySet() {
		return new AbstractByteSet() {

			public boolean contains(final byte k) {
				return containsKey(k);
			}

			public int size() {
				return AbstractByte2ObjectMap.this.size();
			}
			public void clear() {
				AbstractByte2ObjectMap.this.clear();
			}

			public ByteIterator iterator() {
				return new AbstractByteIterator() {
					final ObjectIterator<Map.Entry<Byte, V>> i = entrySet()
							.iterator();
					@Override
					public byte nextByte() {
						return ((Byte2ObjectMap.Entry<V>) i.next())
								.getByteKey();
					};
					@Override
					public boolean hasNext() {
						return i.hasNext();
					}
					@Override
					public void remove() {
						i.remove();
					}
				};
			}
		};
	}

	/**
	 * Returns a type-specific-set view of the values of this map.
	 *
	 * <P>
	 * The view is backed by the set returned by {@link #entrySet()}. Note that
	 * <em>no attempt is made at caching the result of this method</em>, as this
	 * would require adding some attributes that lightweight implementations
	 * would not need. Subclasses may easily override this policy by calling
	 * this method and caching the result, but implementors are encouraged to
	 * write more efficient ad-hoc implementations.
	 *
	 * @return a set view of the values of this map; it may be safely cast to a
	 *         type-specific interface.
	 */

	public ObjectCollection<V> values() {
		return new AbstractObjectCollection<V>() {

			public boolean contains(final Object k) {
				return containsValue(k);
			}

			public int size() {
				return AbstractByte2ObjectMap.this.size();
			}
			public void clear() {
				AbstractByte2ObjectMap.this.clear();
			}

			public ObjectIterator<V> iterator() {
				return new AbstractObjectIterator<V>() {
					final ObjectIterator<Map.Entry<Byte, V>> i = entrySet()
							.iterator();

					/**
					 * {@inheritDoc}
					 * 
					 * @deprecated Please use the corresponding type-specific
					 *             method instead.
					 */
					@Deprecated
					public V next() {
						return ((Byte2ObjectMap.Entry<V>) i.next()).getValue();
					};

					public boolean hasNext() {
						return i.hasNext();
					}
				};
			}
		};
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	public ObjectSet<Map.Entry<Byte, V>> entrySet() {
		return (ObjectSet) byte2ObjectEntrySet();
	}

	/**
	 * Returns a hash code for this map.
	 *
	 * The hash code of a map is computed by summing the hash codes of its
	 * entries.
	 *
	 * @return a hash code for this map.
	 */

	public int hashCode() {
		int h = 0, n = size();
		final ObjectIterator<? extends Map.Entry<Byte, V>> i = entrySet()
				.iterator();

		while (n-- != 0)
			h += i.next().hashCode();
		return h;
	}

	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (!(o instanceof Map))
			return false;

		Map<?, ?> m = (Map<?, ?>) o;
		if (m.size() != size())
			return false;
		return entrySet().containsAll(m.entrySet());
	}

	public String toString() {
		final StringBuilder s = new StringBuilder();
		final ObjectIterator<? extends Map.Entry<Byte, V>> i = entrySet()
				.iterator();
		int n = size();
		Byte2ObjectMap.Entry<V> e;
		boolean first = true;

		s.append("{");

		while (n-- != 0) {
			if (first)
				first = false;
			else
				s.append(", ");

			e = (Byte2ObjectMap.Entry<V>) i.next();

			s.append(String.valueOf(e.getByteKey()));
			s.append("=>");

			if (this == e.getValue())
				s.append("(this map)");
			else

				s.append(String.valueOf(e.getValue()));
		}

		s.append("}");
		return s.toString();
	}

}

==> fastutil-7.2.0-sources.jar: it/unimi/dsi/fastutil/bytes/AbstractByte2ObjectMap.java <==
/*
	* Copyright (C) 2002-2017 Sebastiano Vigna
	*
	* Licensed under the Apache License, Version 2.0 (the "License");
	* you may not use this file except in compliance with the License.
	* You may obtain a copy of the License at
	*
	*     http://www.apache.org/licenses/LICENSE-2.0
	*
	* Unless required by applicable law or agreed to in writing, software
	* distributed under the License is distributed on an "AS IS" BASIS,
	* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	* See the License for the specific language governing permissions and
	* limitations under the License.
	*/
package it.unimi.dsi.fastutil.bytes;

import it.unimi.dsi.fastutil.objects.ObjectCollection;
import it.unimi.dsi.fastutil.objects.AbstractObjectCollection;
import it.unimi.dsi.fastutil.objects.ObjectIterator;
import it.unimi.dsi.fastutil.objects.AbstractObjectIterator;
import it.unimi.dsi.fastutil.objects.ObjectSet;
import java.util.Iterator;
import java.util.Map;

/**
 * An abstract class providing basic methods for maps implementing a
 * type-specific interface.
 *
 * <P>
 * Optional operations just throw an {@link UnsupportedOperationException}.
 * Generic versions of accessors delegate to the corresponding type-specific
 * counterparts following the interface rules (they take care of returning
 * {@code null} on a missing key).
 *
 * <P>
 * As a further help, this class provides a {@link BasicEntry BasicEntry} inner
 * class that implements a type-specific version of {@link java.util.Map.Entry};
 * it is particularly useful for those classes that do not implement their own
 * entries (e.g., most immutable maps).
 */
public abstract class AbstractByte2ObjectMap<V> extends AbstractByte2ObjectFunction<V> implements Byte2ObjectMap<V>, java.io.Serializable {
	private static final long serialVersionUID = -4940583368468432370L;

	protected AbstractByte2ObjectMap() {
	}

	@Override
	public boolean containsValue(Object v) {
		return values().contains(v);
	}

	@Override
	public boolean containsKey(byte k) {
		ObjectIterator<Byte2ObjectMap.Entry<V>> i = byte2ObjectEntrySet().iterator();
		while (i.hasNext())
			if (i.next().getByteKey() == k) return true;
		return false;
	}

	/**
	 * Puts all pairs in the given map. If the map implements the interface of
	 * this map, it uses the faster iterators.
	 *
	 * @param m
	 *            a map.
	 */
	@SuppressWarnings({ "unchecked", "deprecation" })
	public void putAll(Map<? extends Byte, ? extends V> m) {
		int n = m.size();
		final Iterator<? extends Map.Entry<? extends Byte, ? extends V>> i = m.entrySet().iterator();
		if (m instanceof Byte2ObjectMap) {
			Byte2ObjectMap.Entry<? extends V> e;
			while (n-- != 0) {
				e = (Byte2ObjectMap.Entry<? extends V>) i.next();
				put(e.getByteKey(), e.getValue());
			}
		} else {
			Map.Entry<? extends Byte, ? extends V> e;
			while (n-- != 0) {
				e = i.next();
				put(e.getKey(), e.getValue());
			}
		}
	}

	public boolean isEmpty() {
		return size() == 0;
	}

	/**
	 * This class provides a basic but complete type-specific entry class for
	 * all those maps implementations that do not have entries on their own
	 * (e.g., most immutable maps).
	 *
	 * <P>
	 * This class does not implement {@link java.util.Map.Entry#setValue(Object)
	 * setValue()}, as the modification would not be reflected in the base map.
	 */
	public static class BasicEntry<V> implements Byte2ObjectMap.Entry<V> {
		protected byte key;
		protected V value;

		public BasicEntry(final Byte key, final V value) {
			this.key = ((key).byteValue());
			this.value = (value);
		}

		public BasicEntry(final byte key, final V value) {
			this.key = key;
			this.value = value;
		}

		/**
		 * {@inheritDoc}
		 * 
		 * @deprecated Please use the corresponding type-specific method
		 *             instead.
		 */
		@Deprecated
		@Override
		public Byte getKey() {
			return (Byte.valueOf(key));
		}

		@Override
		public byte getByteKey() {
			return key;
		}

		@Override
		public V getValue() {
			return (value);
		}

		@Override
		public V setValue(final V value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public boolean equals(final Object o) {
			if (!(o instanceof Map.Entry)) return false;
			final Map.Entry<?, ?> e = (Map.Entry<?, ?>) o;
			if (e.getKey() == null || !(e.getKey() instanceof Byte)) return false;
			return ((key) == (((((Byte) (e.getKey())).byteValue())))) && ((value) == null ? ((e.getValue())) == null : (value).equals((e.getValue())));
		}

		@Override
		public int hashCode() {
			return (key) ^ ((value) == null ? 0 : (value).hashCode());
		}

		@Override
		public String toString() {
			return key + "->" + value;
		}
	}

	/**
	 * Returns a type-specific-set view of the keys of this map.
	 *
	 * <P>
	 * The view is backed by the set returned by {@link #entrySet()}. Note that
	 * <em>no attempt is made at caching the result of this method</em>, as this
	 * would require adding some attributes that lightweight implementations
	 * would not need. Subclasses may easily override this policy by calling
	 * this method and caching the result, but implementors are encouraged to
	 * write more efficient ad-hoc implementations.
	 *
	 * @return a set view of the keys of this map; it may be safely cast to a
	 *         type-specific interface.
	 */
	@Override
	public ByteSet keySet() {
		return new AbstractByteSet() {
			public boolean contains(final byte k) {
				return containsKey(k);
			}

			public int size() {
				return AbstractByte2ObjectMap.this.size();
			}

			public void clear() {
				AbstractByte2ObjectMap.this.clear();
			}

			public ByteIterator iterator() {
				return new AbstractByteIterator() {
					final ObjectIterator<Map.Entry<Byte, V>> i = entrySet().iterator();

					@Override
					public byte nextByte() {
						return ((Byte2ObjectMap.Entry<V>) i.next()).getByteKey();
					};

					@Override
					public boolean hasNext() {
						return i.hasNext();
					}

					@Override
					public void remove() {
						i.remove();
					}
				};
			}
		};
	}

	/**
	 * Returns a type-specific-set view of the values of this map.
	 *
	 * <P>
	 * The view is backed by the set returned by {@link #entrySet()}. Note that
	 * <em>no attempt is made at caching the result of this method</em>, as this
	 * would require adding some attributes that lightweight implementations
	 * would not need. Subclasses may easily override this policy by calling
	 * this method and caching the result, but implementors are encouraged to
	 * write more efficient ad-hoc implementations.
	 *
	 * @return a set view of the values of this map; it may be safely cast to a
	 *         type-specific interface.
	 */
	@Override
	public ObjectCollection<V> values() {
		return new AbstractObjectCollection<V>() {
			public boolean contains(final Object k) {
				return containsValue(k);
			}

			public int size() {
				return AbstractByte2ObjectMap.this.size();
			}

			public void clear() {
				AbstractByte2ObjectMap.this.clear();
			}

			public ObjectIterator<V> iterator() {
				return new AbstractObjectIterator<V>() {
					final ObjectIterator<Map.Entry<Byte, V>> i = entrySet().iterator();

					/**
					 * {@inheritDoc}
					 * 
					 * @deprecated Please use the corresponding type-specific
					 *             method instead.
					 */
					@Deprecated
					public V next() {
						return ((Byte2ObjectMap.Entry<V>) i.next()).getValue();
					};

					public boolean hasNext() {
						return i.hasNext();
					}
				};
			}
		};
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @deprecated Please use the corresponding type-specific method instead.
	 */
	@Deprecated
	@Override
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public ObjectSet<Map.Entry<Byte, V>> entrySet() {
		return (ObjectSet) byte2ObjectEntrySet();
	}

	/**
	 * Returns a hash code for this map.
	 *
	 * The hash code of a map is computed by summing the hash codes of its
	 * entries.
	 *
	 * @return a hash code for this map.
	 */
	@Override
	public int hashCode() {
		int h = 0, n = size();
		final ObjectIterator<? extends Map.Entry<Byte, V>> i = entrySet().iterator();
		while (n-- != 0)
			h += i.next().hashCode();
		return h;
	}

	@Override
	public boolean equals(Object o) {
		if (o == this) return true;
		if (!(o instanceof Map)) return false;
		Map<?, ?> m = (Map<?, ?>) o;
		if (m.size() != size()) return false;
		return entrySet().containsAll(m.entrySet());
	}

	@Override
	public String toString() {
		final StringBuilder s = new StringBuilder();
		final ObjectIterator<? extends Map.Entry<Byte, V>> i = entrySet().iterator();
		int n = size();
		Byte2ObjectMap.Entry<V> e;
		boolean first = true;
		s.append("{");
		while (n-- != 0) {
			if (first) first = false;
			else s.append(", ");
			e = (Byte2ObjectMap.Entry<V>) i.next();
			s.append(String.valueOf(e.getByteKey()));
			s.append("=>");
			if (this == e.getValue()) s.append("(this map)");
			else s.append(String.valueOf(e.getValue()));
		}
		s.append("}");
		return s.toString();
	}
}
